/*******************************************************************************
 * FileName:        myTasks.c
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Controls
 * Created:         August 20, 2015, 11:39 AM
 ******************************************************************************/
#include "echo.h"

/*******************************************************************************
 * Desc sets up the LEDs, SW0, system config, Uart1, and enables interrupts
 * Parm ulWantedBaud - buad rate EX: 115200ul
 * Parm uxQueueLength - size of TX and RX queues
 * Return None
 ******************************************************************************/
void SetupHardware( unsigned long ulWantedBaud, unsigned portBASE_TYPE uxQueueLength)
{
    mJTAGPortEnable(0);                     //Disable JTAG and reclaim IO

    LEDclr();                               //Clear LED registers
    LEDtris();                              //Set LED pins to outputs

    SW0_TRIS();                             //set push button to input

    //Set the system and peripheral bus speeds and enable the program cache
    SYSTEMConfigPerformance( configCPU_CLOCK_HZ );
    mOSCSetPBDIV( OSC_PB_DIV_2 );

    //open Uart1 with Tx and Rx interrupts enabled
    xSerialPortInitMinimal(ulWantedBaud, uxQueueLength);

    //Enable global interrupt handling.
    INTEnableSystemMultiVectoredInt();

    portDISABLE_INTERRUPTS();

}

/*******************************************************************************
 * Desc Task to flash LED1 at a set rate
 * Parm pvParameters - not used, but needed for function pointer
 * Return None
 ******************************************************************************/
void vLEDFlashTask( void *pvParameters )
{
    Params Prams = *(Params *)pvParameters;
    portTickType xLastWakeTime = xTaskGetTickCount();

    while(1)
    {
        LATFINV = Prams.LED;
        vTaskDelayUntil(&xLastWakeTime,Prams.Period);
    }
}

/*******************************************************************************
 * Desc Task to debounce and check button state. If button is zero (pressed)
 *      resume the task, else suspend the task.
 * Parm pvParameters - not used, but needed for function pointer
 * Return None
 ******************************************************************************/
void vButtonCheckTask( void *pvParameters )
{
    static UINT8 count;                                   //for debounce
    portTickType xLastWakeTime = xTaskGetTickCount();

    //enter service loop
    while(1)
    {
        if (SW0_IO)
        {
            count++;
            if(count==DEBOUNCECOUNTS)
            {
                count = 0;
                vTaskSuspend(LED3_Hndl);
                LATFCLR = 0x02;
            }
        }
        else
        {
            count++;
            if(count==DEBOUNCECOUNTS)
            {
                count = 0;
                vTaskResume(LED3_Hndl);
            }
        }

        vTaskDelayUntil(&xLastWakeTime,*(portTickType *)pvParameters );
    }
}

/*******************************************************************************
 * Desc This function echos the RX buffer to the TX buffer. If semaphore avalible,
 * copy string from RX to TX and enable TX interrupt to start transmition.
 * Parm pvParameters - not used, but needed for function pointer
 * Return None
 ******************************************************************************/
void RxQueueToTxQueue( void *pvParameters )
{
    unsigned char ucChar;
    extern xSemaphoreHandle xRXreturnSemaphore;
    
    while(1)
    {
        if(xSemaphoreTake(xRXreturnSemaphore, portMAX_DELAY ) == pdTRUE)
        {           
            while(xSerialGetChar(&ucChar, 100) == pdTRUE)
                xSerialPutChar(ucChar, 100 );
            IEC0SET = _IEC0_U1TXIE_MASK;            
        }
    }
}

/*******************************************************************************
 * Desc Sents a string to Uart1 every 3 sec.
 * Parm pvParameters - not used, but needed for function pointer
 * Return None
 ******************************************************************************/
void SendStringUart1( void *pvParameters )
{
    while(1)
    {
        vTaskDelay(3000);
        xSerialPutString("Goodby World\r\n", 500);

    }
}